/*
 * C
 *
 * Copyright 2013 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include "microej.h"
#include "native_monitor.h"
#include "sni.h"

// RTX
#include <RTL.h>

typedef struct {
	int32_t javaRequester;
	OS_MUT osMonitor;
} NativeMonitor;

#define NUMBER_OF_MONITORS 3
static int32_t next_monitorIndex = 0;
static NativeMonitor monitors[NUMBER_OF_MONITORS];

/*
 * Functions that implement the Java natives
 */

int32_t Java_com_is2t_examples_NativeMonitor_newNativeMonitor(void) {
	return new_native_monitor();
}

int32_t Java_com_is2t_examples_NativeMonitor_tryEnter(int32_t monitorID) {
	OS_RESULT result;
	NativeMonitor* monitor = &monitors[monitorID];
	monitor->javaRequester = SNI_getCurrentJavaThreadID();
	result = os_mut_wait(&monitor->osMonitor, 0);
	if (result == OS_R_TMO) {
		SNI_suspendCurrentJavaThread(0);
		return JFALSE;
	} else {
		monitor->javaRequester = JNULL;
		return JTRUE;
	}
}

void Java_com_is2t_examples_NativeMonitor_exit__I(int32_t monitorID) {
	NativeMonitor* monitor = &monitors[monitorID];
	os_mut_release(&monitor->osMonitor);
}

/*
 * Functions that implement the C API
 */

int32_t new_native_monitor(void) {
	int32_t id;
	NativeMonitor* monitor;
	if (next_monitorIndex == NUMBER_OF_MONITORS) {
		return -1;
	}
	id = next_monitorIndex++;
	monitor = &monitors[id];
	os_mut_init(&monitor->osMonitor);
	return id;
}

void native_monitor_enter(int32_t monitorID) {
	NativeMonitor* monitor = &monitors[monitorID];
	OS_RESULT result = os_mut_wait(&monitor->osMonitor, 0xFFFF);
}

void native_monitor_exit(int32_t monitorID) {
	int32_t javaRequester;
	NativeMonitor* monitor = &monitors[monitorID];
	os_mut_release(&monitor->osMonitor);
	javaRequester = monitor->javaRequester;
	if (javaRequester != JNULL) {
		int32_t result = SNI_resumeJavaThread(javaRequester);
	}
}
